# Alantech Roman Numeral Converter
# roman_to_decimal, decimal_to_roman (grunt-work functions)
# (c) Alan Gilfoy, 2007

def decimal_to_roman(decimal_input):
    '''
    convert base-10 (decimal) integers less than 4000 into Roman numerals

    >>> decimal_to_roman(42)
    'XLII'
    >>> decimal_to_roman(1337)
    'MCCCXXXVII'
    >>> decimal_to_roman(12)
    'XII'
    >>> decimal_to_roman(7)
    'VII'
    >>> decimal_to_roman(1941)
    'MCMXLI'
    >>> decimal_to_roman(11)
    'XI'
    >>> decimal_to_roman(22)
    'XXII'
    >>> decimal_to_roman(1963)
    'MCMLXIII'
    >>> decimal_to_roman(3)
    'III'
    >>> decimal_to_roman(10)
    'X'
    >>> decimal_to_roman(2007)
    'MMVII'
    '''

    roman_result = ""
    decimal_input = decimal_input

    try:
        decimal_input = int(decimal_input)

    except ValueError:
        #Integers, please.
        raise ValueError, "Number input must be an integer."
        

    dtr_conv = True
    while dtr_conv:

        if decimal_input > 3999:
            raise ValueError, "This program can only convert into Roman numerals less than 4000."

        if decimal_input < 0:
            raise ValueError, "There's no such thing as negative Roman numerals."

        #This section of code works to "process" the thousands-place of the input
        #by subtracting 1000 from the input an appropriate amoutn of times'
        #and adding the Roman numeral M an appropriate amount of times
            
        if decimal_input >= 1000:
            decimal_input -= 1000 # Python shorthand for setting decimal_input equal to itself minus 1000
            roman_result += "M" # Python shorthand for setting roman_result equal to itself plus "M"
            continue

        #to "process the hundreds-place of the input
        if decimal_input >= 900 and decimal_input < 1000:
            decimal_input -= 900
            roman_result += "CM"

        if decimal_input >= 500 and decimal_input < 900:
            decimal_input -= 500
            roman_result += "D"

        if decimal_input >= 400 and decimal_input < 500:
            decimal_input -= 400
            roman_result += "CD"

        if decimal_input >= 100 and decimal_input < 400:
            decimal_input -= 100
            roman_result += "C"
            continue

        if decimal_input >= 90 and decimal_input < 100:
            decimal_input -= 90
            roman_result += "XC"

        if decimal_input >= 50 and decimal_input < 90:
            decimal_input -= 50
            roman_result += "L"

        if decimal_input >= 40 and decimal_input < 50:
            decimal_input -= 40
            roman_result += "XL"
                    
        #X-loop
        if decimal_input >= 10 and decimal_input < 40:
            decimal_input -= 10
            roman_result += "X"
            continue

        if decimal_input >= 5 and decimal_input < 9:
            decimal_input -= 5
            roman_result += "V"

        if decimal_input == 9:
            decimal_input -= 9
            roman_result += "IX"

        if decimal_input == 4:
            decimal_input -= 4
            roman_result += "IV"

        if decimal_input >= 1:
            decimal_input -= 1
            roman_result += "I"
            continue

        #workaround - normally, I would use a loop similar to the X-loop (see it labeled above), except with 1 and "I")
        #however, although the X-loop was working properly for 20 and 30, and the M-loop for 2000 and 3000,
        #the old I-loop only added one I to the result string, even if 2 or 3 was being inputted as 'decimal_input'.

        return roman_result

def roman_to_decimal(raw_roman_input):
    '''
    convert Roman numeral into a decimal (base-10) number

    >>> roman_to_decimal("III")
    3
    >>> roman_to_decimal("IV")
    4
    >>> roman_to_decimal("IX")
    9
    >>> roman_to_decimal("XIX")
    19
    >>> roman_to_decimal("XLIX")
    49
    >>> roman_to_decimal("DCLXVI")
    666
    >>> roman_to_decimal("MMMDCCCLXXXVIII")
    3888
    >>> roman_to_decimal("MCMXCIX")
    1999
    >>> roman_to_decimal("CD")
    400
    >>> roman_to_decimal("MDCX")
    1610
    >>> roman_to_decimal("MCDX")
    1400
    >>> roman_to_decimal("MMXVIII")
    2018
    >>> roman_to_decimal("MCCCXXXVII")
    1337
    '''
     
    decimal_result = 0

    roman_input = raw_roman_input.upper()
    count = len(roman_input)

    #input prompt in main program asks for capital letters
    #but this line of code fixes one possible error (noncapitalized Roman numeral letters).
    # ( i, v, x, l, c, d, and/or m)

    if roman_input == "ACDC":
        decimal_result = "Wait a second... Incorrect Input! They produced good rock songs, but I don't think they produced any Roman numerals."

    rtd_conv = True
    while rtd_conv:

        if count >= 2:
            if roman_input.startswith("M"):
                decimal_result += 1000
                roman_input = roman_input[1:]
                count -= 1
                continue
                
            if roman_input.startswith("D"):
                decimal_result += 500
                roman_input = roman_input[1:]
                count -= 1
                
            if roman_input.startswith("C"):
                if roman_input[1:2] not in "D" and roman_input[1:2] not in "M":
                    decimal_result += 100
                    roman_input = roman_input[1:]
                    count -= 1
                    continue
                if roman_input[1:2] == "D":
                    decimal_result += 400
                    roman_input = roman_input[2:]
                if roman_input[1:2] == "M":
                    decimal_result += 900
                    roman_input = roman_input[2:]

            if roman_input.startswith("L"):
                decimal_result += 50
                roman_input = roman_input[1:]
                count -= 1
                    
            if roman_input.startswith("X"):
                if roman_input[1:2] not in "L" and roman_input[1:2] not in "C":
                    decimal_result += 10
                    roman_input = roman_input[1:]
                    count -= 1
                    continue
                if roman_input[1:2] == "L":
                    decimal_result += 40
                    roman_input = roman_input[2:]
                if roman_input[1:2] == "C":
                    decimal_result += 90
                    roman_input = roman_input[2:]

            if roman_input.startswith("V"):
                decimal_result += 5
                roman_input = roman_input[1:]
                count -= 1

            if roman_input.startswith("I"):
                if roman_input[1:2] not in "V" and roman_input[1:2] not in "X":
                    decimal_result += 1
                    roman_input = roman_input[1:]
                    count -= 1
                    continue
                if roman_input[1:2] == "V":
                    decimal_result += 4
                    roman_input = roman_input[2:]
                if roman_input[1:2] == "X":
                    decimal_result += 9
                    roman_input = roman_input[2:]
                    
        if count == 1:
            if roman_input == "M":
                decimal_result += 1000
                roman_input = ""
            if roman_input == "D":
                decimal_result += 500
                roman_input = ""
            if roman_input == "C":
                decimal_result += 100
                roman_input = ""
            if roman_input == "L":
                decimal_result += 50
                roman_input = ""
            if roman_input == "X":
                decimal_result += 10
                roman_input = ""
            if roman_input == "V":
                decimal_result += 5
                roman_input = ""
            if roman_input == "I":
                decimal_result += 1
                roman_input = ""
                
        return decimal_result
    
if __name__ == '__main__':
    print "This module contains functions to convert Roman numerals to their decimal (base-10) equivalent, and vice versa."
    print "This is a module, a component file used by the RomanNumeralConverter program."
    print "To run a conversion, run RomanNumeralConverter, which should be in the same directory as this file."
